@@ -2,16 +2,18 @@
 /* Copyright (C) 1998 Redhat Software Inc. 
  * Authors: Jonathan Blandford <jrb@redhat.com>
  */
-#include <config.h>
+
+#ifdef HAVE_CONFIG_H
+#   include <config.h>
+#endif
+
 #include "capplet-widget.h"
-#include <gnome.h>
-#include <libgnomeui/gnome-window-icon.h>
+#include "gnome.h"
 #include <string.h>
 #include <sys/types.h>
 #include <dirent.h>
 #include <regex.h>
 #include <ctype.h>
-#include <locale.h>
 #include "mime-data.h"
 #include "mime-info.h"
 #include "edit-window.h"
@@ -67,21 +69,7 @@ cancel_callback ()
 static void
 help_callback ()
 {
-  gchar *tmp;
-  
-  tmp = gnome_help_file_find_file ("control-center", "doc-handlers.html#GCCMIME");
-  if (tmp) {
-          gnome_help_goto(0, tmp);
-          g_free(tmp);
-  } else {
-          GtkWidget *mbox;
-          
-          mbox = gnome_message_box_new(_("No help is available/installed for these settings. Please make sure you\nhave the Control Center documentation installed on your system."),
-				 GNOME_MESSAGE_BOX_ERROR,
-				 _("Close"), NULL);
-    
-          gtk_widget_show(mbox);
-  }
+        /* Sigh... empty as always */
 }
 
 static void
@@ -124,8 +112,10 @@ init_mime_capplet ()
                            GTK_SIGNAL_FUNC(hide_edit_window), NULL);
         gtk_signal_connect(GTK_OBJECT(capplet), "page_shown",
                            GTK_SIGNAL_FUNC(show_edit_window), NULL);
+#if 0
         gtk_signal_connect(GTK_OBJECT(capplet), "help",
                            GTK_SIGNAL_FUNC(help_callback), NULL);
+#endif
 }
 
 int
@@ -133,14 +123,12 @@ main (int argc, char **argv)
 {
         int init_results;
 
-				setlocale(LC_ALL, "");
         bindtextdomain (PACKAGE, GNOMELOCALEDIR);
         textdomain (PACKAGE);
 
         init_results = gnome_capplet_init("mime-type-capplet", VERSION,
                                           argc, argv, NULL, 0, NULL);
 
-        gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-ccmime.png");
 	if (init_results < 0) {
                 exit (0);
 	}
