@@ -22,7 +22,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+#   include "config.h"
 #endif
 
 #include <gnome.h>
@@ -341,7 +341,9 @@ screensaver_prefs_dialog_destroy (ScreensaverPrefsDialog *dialog)
 static void
 free_set_cb (gchar *key, PrefsDialogWidgetSet *set, gpointer data) 
 {
-	g_list_free (set->widgets);
+	if (!set->alias)
+		g_list_free (set->widgets);
+
 	g_free (set);
 }
 
@@ -378,6 +380,8 @@ set_widgets_sensitive (GTree *widget_db,
 		set = g_tree_lookup (widget_db, str);
 		if (!set) continue;
 
+		set->enabled = s;
+
 		for (node = set->widgets; node; node = node->next)
 			gtk_widget_set_sensitive (GTK_WIDGET (node->data), s);
 	}
@@ -450,7 +454,7 @@ write_boolean (xmlNodePtr argument_data, GTree *widget_db)
 	if (!(id = xmlGetProp (argument_data, "id"))) return;
 	set = g_tree_lookup (widget_db, id);
 
-	if (!set || !GTK_WIDGET_IS_SENSITIVE (set->value_widget)) return NULL;
+	if (!set || !set->enabled) return NULL;
 
 	if (gtk_toggle_button_get_active 
 	    (GTK_TOGGLE_BUTTON (set->value_widget)))
@@ -472,7 +476,7 @@ write_number (xmlNodePtr argument_data, GTree *widget_db)
 	if (!(id = xmlGetProp (argument_data, "id"))) return;
 	set = g_tree_lookup (widget_db, id);
 
-	if (!set || !GTK_WIDGET_IS_SENSITIVE (set->value_widget)) return NULL;
+	if (!set || !set->enabled) return NULL;
 
 	if (GTK_IS_RANGE (set->value_widget))
 		adjustment = gtk_range_get_adjustment 
@@ -521,25 +525,58 @@ write_select (xmlNodePtr argument_data, GTree *widget_db)
 	GList *menu_item;
 	char *id;
 
-	node = argument_data->childs;
 	if (!(id = xmlGetProp (argument_data, "id"))) return;
 	set = g_tree_lookup (widget_db, id);
 
-	if (!set || !GTK_WIDGET_IS_SENSITIVE (set->value_widget)) return NULL;
+	if (!set || !set->enabled) return NULL;
 	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (set->value_widget));
 	menu_item = GTK_MENU_SHELL (menu)->children;
 	active = gtk_menu_get_active (GTK_MENU (menu));
 
-	while (node && menu_item) {
+	for (node = argument_data->childs; node && menu_item;
+	     node = node->next) 
+	{
+		if (!xmlGetProp (node, "id")) continue;
+
 		if (active == menu_item->data)
 			return xmlGetProp (node, "arg-set");
 
-		node = node->next; menu_item = menu_item->next; i++;
+		menu_item = menu_item->next; i++;
 	}
 
 	return NULL;
 }
 
+static gchar *
+write_string (xmlNodePtr argument_data, GTree *widget_db) 
+{
+	PrefsDialogWidgetSet *set;
+	gchar *str;
+	char *id, *arg, *ret_str;
+	char *pos;
+
+	if (!(id = xmlGetProp (argument_data, "id"))) return;
+	set = g_tree_lookup (widget_db, id);
+
+	if (!set || !set->enabled) return NULL;
+
+	str = gtk_entry_get_text (GTK_ENTRY (set->value_widget));
+	arg = xmlGetProp (argument_data, "arg");
+
+	if (!arg) return NULL;
+	arg = g_strdup (arg);
+
+	pos = strchr (arg, '%');
+
+	if (!pos) return arg;
+	*pos = '\0';
+
+	ret_str = g_strdup_printf ("%s\"%s\"%s", arg, (int) str, pos + 1);
+	g_free (arg);
+
+	return ret_str;
+}
+
 /* write_command_line
  *
  * Scan through XML nodes in the argument definition file and write
@@ -553,23 +590,41 @@ write_command_line (gchar *name, xmlNodePtr argument_data, GTree *widget_db)
 	xmlNodePtr node;
 	gchar *arg, *ret;
 	gboolean flag = FALSE;
+	gboolean free_v = FALSE;
 
 	line = g_string_new (name);
 	node = argument_data->childs;
 
 	for (node = argument_data->childs; node; node = node->next) {
-		if (!strcmp (node->name, "boolean"))
+		if (!strcmp (node->name, "boolean")) {
 			arg = write_boolean (node, widget_db);
-		else if (!strcmp (node->name, "number"))
+			free_v = FALSE;
+		}
+		else if (!strcmp (node->name, "number")) {
 			arg = write_number (node, widget_db);
-		else if (!strcmp (node->name, "select"))
+			free_v = TRUE;
+		}
+		else if (!strcmp (node->name, "select")) {
 			arg = write_select (node, widget_db);
-		else if (!strcmp (node->name, "command"))
+			free_v = FALSE;
+		}
+		else if (!strcmp (node->name, "string") ||
+			 !strcmp (node->name, "file")) 
+		{
+			arg = write_string (node, widget_db);
+			free_v = TRUE;
+		}
+		else if (!strcmp (node->name, "command")) {
 			arg = xmlGetProp (node, "arg");
-		else if (!strcmp (node->name, "hgroup"))
+			free_v = FALSE;
+		}
+		else if (!strcmp (node->name, "hgroup")) {
 			arg = write_command_line (NULL, node, widget_db);
-		else
+			free_v = TRUE;
+		} else {
 			arg = NULL;
+			free_v = FALSE;
+		}
 
 		if (arg) {
 			if (*arg && (name || flag)) 
@@ -577,9 +632,7 @@ write_command_line (gchar *name, xmlNodePtr argument_data, GTree *widget_db)
 			g_string_append (line, arg);
 			flag = TRUE;
 
-			if (!strcmp (node->name, "number") ||
-			    !strcmp (node->name, "hgroup")) 
-				g_free (arg);
+			if (free_v) g_free (arg);
 		}
 	}
 
@@ -595,11 +648,10 @@ write_command_line (gchar *name, xmlNodePtr argument_data, GTree *widget_db)
 static GScanner *
 read_command_line (char *command_line) 
 {
-	char **args;
 	int i;
 	GScanner *cli_db;
 	static GScannerConfig config;
-	char *arg, *value;
+	char *arg, *value, *argpos, *valpos, *endpos;
 
 	config.cset_skip_characters = " \t\n";
 	config.cset_identifier_first = "abcdefghijklmnopqrstuvwxyz";
@@ -610,28 +662,73 @@ read_command_line (char *command_line)
 	cli_db = g_scanner_new (&config);
 	g_scanner_set_scope (cli_db, 0);
 
-	args = g_strsplit (command_line, " ", -1);
-
 	g_scanner_scope_add_symbol (cli_db, 0, "and", SYMBOL_AND);
 	g_scanner_scope_add_symbol (cli_db, 0, "or", SYMBOL_OR);
 	g_scanner_scope_add_symbol (cli_db, 0, "not", SYMBOL_NOT);
 
-	for (i = 0; args[i]; i++) {
-		if (args[i][0] == '-') {
-			arg = g_strdup (args[i] + 1);
+	command_line = g_strdup (command_line);
+	argpos = command_line;
 
-			if (args[i + 1] && args[i + 1][0] != '-') {
-				value = g_strdup (args[i + 1]);
-				i++;
+	while (argpos && *argpos) {
+		if (*argpos == '-') {
+			valpos = strchr (argpos, ' ');
+			if (valpos) *valpos = '\0';
+			arg = g_strdup (argpos + 1);
+
+			if (valpos) {
+				valpos++;
+				while (isspace (*valpos)) valpos++;
+
+				if (*valpos == '\"') {
+					endpos = strchr (valpos + 1, '\"');
+
+					if (endpos) {
+						*endpos = '\0';
+						endpos++;
+					}
+
+					value = g_strdup (valpos + 1);
+
+					if (endpos) {
+						endpos++;
+						while (isspace (*endpos)) 
+							endpos++;
+					}
+				} else if (*valpos != '-' || 
+					   isdigit(valpos[1])) 
+				{
+					endpos = strchr (valpos, ' ');
+					if (endpos) *endpos = '\0';
+					value = g_strdup (valpos);
+
+					if (endpos) {
+						endpos++;
+						while (isspace (*endpos)) 
+							endpos++;
+					}
+				} else {
+					value = (char *) 1;
+					endpos = valpos;
+				}
+
+				if (endpos)
+					argpos = endpos;
+				else
+					argpos = NULL;
 			} else {
 				value = (char *) 1;
+				argpos = NULL;
 			}
 
 			g_scanner_scope_add_symbol (cli_db, 0, arg, value);
+		} else {
+			argpos = strchr (argpos, ' ');
+			if (argpos)
+				while (isspace (*argpos)) argpos++;
 		}
 	}
 
-	g_strfreev (args);
+	g_free (command_line);
 
 	return cli_db;
 }
@@ -713,6 +810,8 @@ get_spinbutton (xmlNodePtr node, GtkWidget **widget)
 	spinbutton = gtk_spin_button_new (adjustment, 1.0, 0);
 	
 	set = g_new0 (PrefsDialogWidgetSet, 1);
+	set->alias = FALSE;
+	set->enabled = TRUE;
 	set->value_widget = spinbutton;
 
 	if (label_str) {
@@ -756,6 +855,8 @@ get_check_button (ScreensaverPrefsDialog *dialog, xmlNodePtr node,
 	checkbutton = gtk_check_button_new_with_label (label);
 
 	set = g_new0 (PrefsDialogWidgetSet, 1);
+	set->alias = FALSE;
+	set->enabled = TRUE;
 	set->widgets = g_list_append (NULL, checkbutton);
 	set->value_widget = checkbutton;
 
@@ -782,7 +883,6 @@ get_select_widget (ScreensaverPrefsDialog *dialog, xmlNodePtr select_data,
 	xmlNodePtr node;
 
 	g_return_val_if_fail (widget != NULL, NULL);
-	g_return_val_if_fail (node != NULL, NULL);
 
 	label_str = xmlGetProp (select_data, "label");
 
@@ -790,6 +890,8 @@ get_select_widget (ScreensaverPrefsDialog *dialog, xmlNodePtr select_data,
 	menu = gtk_menu_new ();
 	
 	set = g_new0 (PrefsDialogWidgetSet, 1);
+	set->alias = FALSE;
+	set->enabled = TRUE;
 	set->value_widget = option_menu;
 
 	gtk_object_set_data (GTK_OBJECT (option_menu), "dialog", dialog);
@@ -839,6 +941,102 @@ get_select_widget (ScreensaverPrefsDialog *dialog, xmlNodePtr select_data,
 	return set;
 }
 
+/* Form a GtkFileEntry from a string value */
+
+static PrefsDialogWidgetSet *
+get_file_entry (ScreensaverPrefsDialog *dialog, xmlNodePtr node, 
+		GtkWidget **widget) 
+{
+	char *label_str, *default_str;
+	GtkWidget *hbox, *label, *entry;
+	PrefsDialogWidgetSet *set;
+
+	g_return_val_if_fail (widget != NULL, NULL);
+	g_return_val_if_fail (node != NULL, NULL);
+
+	label_str = xmlGetProp (node, "label");
+	default_str = xmlGetProp (node, "default");
+
+	entry = gnome_file_entry_new (NULL, NULL);
+
+	if (default_str)
+		gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (entry), 
+						   default_str);
+
+	set = g_new0 (PrefsDialogWidgetSet, 1);
+	set->alias = FALSE;
+	set->enabled = TRUE;
+	set->value_widget = 
+		gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (entry));
+
+	if (label_str) {
+		hbox = gtk_hbox_new (FALSE, 5);
+		label = gtk_label_new (label_str);
+		gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+		gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox), entry, 
+				    FALSE, TRUE, 0);
+
+		set->widgets = g_list_append (NULL, hbox);
+		g_list_append (set->widgets, label);
+		g_list_append (set->widgets, entry);
+
+		*widget = hbox;
+	} else {
+		set->widgets = g_list_append (NULL, entry);
+		*widget = entry;
+	}
+
+	return set;
+}
+
+/* Form a GtkEntry from a string value */
+
+static PrefsDialogWidgetSet *
+get_entry (ScreensaverPrefsDialog *dialog, xmlNodePtr node, 
+	   GtkWidget **widget) 
+{
+	char *label_str, *default_str;
+	GtkWidget *hbox, *label, *entry;
+	PrefsDialogWidgetSet *set;
+
+	g_return_val_if_fail (widget != NULL, NULL);
+	g_return_val_if_fail (node != NULL, NULL);
+
+	label_str = xmlGetProp (node, "label");
+	default_str = xmlGetProp (node, "default");
+
+	entry = gtk_entry_new ();
+
+	if (default_str)
+		gtk_entry_set_text (GTK_ENTRY (entry), default_str);
+
+	set = g_new0 (PrefsDialogWidgetSet, 1);
+	set->alias = FALSE;
+	set->enabled = TRUE;
+	set->value_widget = entry;
+
+	if (label_str) {
+		hbox = gtk_hbox_new (FALSE, 5);
+		label = gtk_label_new (label_str);
+		gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+		gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox), entry, 
+				    TRUE, TRUE, 0);
+
+		set->widgets = g_list_append (NULL, hbox);
+		g_list_append (set->widgets, label);
+		g_list_append (set->widgets, entry);
+
+		*widget = hbox;
+	} else {
+		set->widgets = g_list_append (NULL, entry);
+		*widget = entry;
+	}
+
+	return set;
+}
+
 /* Place a set of widgets that configures a numerical value in the
  * next row of the table and update the row value
  */
@@ -876,6 +1074,8 @@ place_number (GtkTable *table, xmlNodePtr node, gint *row)
 			return NULL;
 
 		set = g_new0 (PrefsDialogWidgetSet, 1);
+		set->alias = FALSE;
+		set->enabled = TRUE;
 		label = gtk_label_new (label_str);
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 		gtk_table_attach (table, label, 0, 3, *row, *row + 1,
@@ -968,10 +1168,10 @@ place_hgroup (ScreensaverPrefsDialog *dialog,
 	      GtkTable *table, GTree *widget_db, 
 	      xmlNodePtr hgroup_data, gint *row) 
 {
-	PrefsDialogWidgetSet *set;
+	PrefsDialogWidgetSet *set, *set1;
 	GtkWidget *hbox, *widget;
 	xmlNodePtr node;
-	gchar *id;
+	gchar *id, *same_as;
 
 	g_return_if_fail (table != NULL);
 	g_return_if_fail (GTK_IS_TABLE (table));
@@ -985,6 +1185,20 @@ place_hgroup (ScreensaverPrefsDialog *dialog,
 		id = xmlGetProp (node, "id");
 		if (!id) continue;
 
+		same_as = xmlGetProp (node, "same-as");
+
+		if (same_as != NULL && *same_as != '\0') {
+			set1 = g_tree_lookup (dialog->widget_db, same_as);
+			if (set1 == NULL) continue;
+			set = g_new0 (PrefsDialogWidgetSet, 1);
+			set->alias = TRUE;
+			set->enabled = TRUE;
+			set->value_widget = set1->value_widget;
+			set->widgets = set1->widgets;
+			g_tree_insert (dialog->widget_db, id, set);
+			continue;
+		}
+
 		if (!strcmp (node->name, "number"))
 			set = get_spinbutton (node, &widget);
 		else if (!strcmp (node->name, "boolean"))
@@ -1030,6 +1244,34 @@ place_select (ScreensaverPrefsDialog *dialog, GtkTable *table,
 	return set;
 }
 
+/* Place a GtkEntry or a GnomeFileEntry in a table */
+
+static PrefsDialogWidgetSet *
+place_entry (ScreensaverPrefsDialog *dialog, GtkTable *table,
+	     xmlNodePtr node, gint *row, gboolean is_file) 
+{
+	PrefsDialogWidgetSet *set;
+	GtkWidget *widget;
+
+	g_return_val_if_fail (table != NULL, NULL);
+	g_return_val_if_fail (GTK_IS_TABLE (table), NULL);
+	g_return_val_if_fail (node != NULL, NULL);
+	g_return_val_if_fail (row != NULL, NULL);
+
+	if (is_file)
+		set = get_file_entry (dialog, node, &widget);
+	else
+		set = get_entry (dialog, node, &widget);
+
+	if (set) {
+		gtk_table_attach (table, widget, 0, 3, *row, *row + 1,
+				  GTK_FILL, 0, 0, 0);
+		*row += 1;
+	}
+
+	return set;
+}
+
 /* Fill a GtkTable with widgets based on the XML description of the
  * screensaver 
  */
@@ -1037,8 +1279,8 @@ place_select (ScreensaverPrefsDialog *dialog, GtkTable *table,
 static void
 populate_table (ScreensaverPrefsDialog *dialog, GtkTable *table) 
 {
-	char *id, *type;
-	PrefsDialogWidgetSet *set;
+	char *id, *type, *same_as;
+	PrefsDialogWidgetSet *set, *set1;
 	xmlNodePtr node;
 	gint row = 0;
 
@@ -1055,6 +1297,20 @@ populate_table (ScreensaverPrefsDialog *dialog, GtkTable *table)
 
 		set = NULL;
 
+		same_as = xmlGetProp (node, "same-as");
+
+		if (same_as != NULL && *same_as != '\0') {
+			set1 = g_tree_lookup (dialog->widget_db, same_as);
+			if (set1 == NULL) continue;
+			set = g_new0 (PrefsDialogWidgetSet, 1);
+			set->alias = TRUE;
+			set->enabled = TRUE;
+			set->value_widget = set1->value_widget;
+			set->widgets = set1->widgets;
+			g_tree_insert (dialog->widget_db, id, set);
+			continue;
+		}
+
 		if (!strcmp (node->name, "number"))
 			set = place_number (table, node, &row);
 		else if (!strcmp (node->name, "boolean"))
@@ -1064,6 +1320,10 @@ populate_table (ScreensaverPrefsDialog *dialog, GtkTable *table)
 				      node, &row);
 		else if (!strcmp (node->name, "select"))
 			set = place_select (dialog, table, node, &row);
+		else if (!strcmp (node->name, "string"))
+			set = place_entry (dialog, table, node, &row, FALSE);
+		else if (!strcmp (node->name, "file"))
+			set = place_entry (dialog, table, node, &row, TRUE);
 		else continue;
 
 		if (set) g_tree_insert (dialog->widget_db, id, set);
@@ -1201,7 +1461,8 @@ read_boolean (GTree *widget_db, xmlNodePtr argument_data, GScanner *cli_db)
 	if (!(id = xmlGetProp (argument_data, "id"))) return;
 	set = g_tree_lookup (widget_db, id);
 
-	if (!set || !set->value_widget) return;
+	if (!set || !set->value_widget || (set->alias && !set->enabled))
+		return;
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (set->value_widget), 
 				      found >= 0);
@@ -1236,12 +1497,17 @@ read_number (GTree *widget_db, xmlNodePtr argument_data, GScanner *cli_db)
 	args = g_strsplit (arg_line, " ", -1);
 
 	arg = g_scanner_scope_lookup_symbol (cli_db, 0, args[0] + 1);
-	if (!arg || arg == (char *) 1) return;
+
+	if (!arg || arg == (char *) 1)
+		arg = xmlGetProp (argument_data, "default");
+	if (!arg)
+		return;
 
 	if (!(id = xmlGetProp (argument_data, "id"))) return;
 	set = g_tree_lookup (widget_db, id);
 
-	if (!set || !set->value_widget) return;
+	if (!set || !set->value_widget || (set->alias && !set->enabled))
+		return;
 
 	from_cli_conv = xmlGetProp (argument_data, "from-cli-conv");
 
@@ -1282,16 +1548,17 @@ read_select (GTree *widget_db, xmlNodePtr argument_data,
 	if (!(id = xmlGetProp (argument_data, "id"))) return;
 	set = g_tree_lookup (widget_db, id);
 
-	if (!set || !set->value_widget) return;
+	if (!set || !set->value_widget || (set->alias && !set->enabled))
+		return;
 
 	menu = gtk_option_menu_get_menu 
 		(GTK_OPTION_MENU (set->value_widget));
 
-	node = argument_data->childs;
-
 	/* Get the index of the selected option */
 
-	while (node) {
+	for (node = argument_data->childs; node; node = node->next) {
+		if (!xmlGetProp (node, "id")) continue;
+
 		found = arg_is_set (node, cli_db);
 
 		if (found > max_found) {
@@ -1299,7 +1566,7 @@ read_select (GTree *widget_db, xmlNodePtr argument_data,
 			max_found = found;
 		}
 
-		node = node->next; i++;
+		i++;
 	}
 
 	/* Enable widgets enabled by selected option and disable
@@ -1307,19 +1574,23 @@ read_select (GTree *widget_db, xmlNodePtr argument_data,
 	 * deselect signals to do the same when an option is selected 
 	 */
 
-	node = argument_data->childs; i = 0;
+	i = 0;
+
+	for (node = argument_data->childs; node; node = node->next) {
+		if (!xmlGetProp (node, "id")) continue;
 
-	while (node) {
 		if (i != set_idx)
 			set_widgets_sensitive (widget_db,
 					       xmlGetProp (node, "enable"),
 					       FALSE);
-		node = node->next; i++;
+		i++;
 	}
 
-	node = argument_data->childs; i = 0;
+	i = 0;
+
+	for (node = argument_data->childs; node; node = node->next) {
+		if (!xmlGetProp (node, "id")) continue;
 
-	while (node) {
 		if (i == set_idx) {
 			gtk_option_menu_set_history 
 				(GTK_OPTION_MENU (set->value_widget), i);
@@ -1329,10 +1600,44 @@ read_select (GTree *widget_db, xmlNodePtr argument_data,
 			break;
 		}
 
-		node = node->next; i++;
+		i++;
 	}
 }
 
+static void
+read_string (GTree *widget_db, xmlNodePtr argument_data, GScanner *cli_db,
+	     gboolean is_file) 
+{
+	PrefsDialogWidgetSet *set;
+	char *arg, *id;
+	char *arg_line;
+	char **args;
+
+	g_return_if_fail (widget_db != NULL);
+	g_return_if_fail (argument_data != NULL);
+	g_return_if_fail (cli_db != NULL);
+
+	arg_line = xmlGetProp (argument_data, "arg");
+	args = g_strsplit (arg_line, " ", -1);
+
+	arg = g_scanner_scope_lookup_symbol (cli_db, 0, args[0] + 1);
+
+	if (!arg || arg == (char *) 1)
+		arg = xmlGetProp (argument_data, "default");
+	if (!arg)
+		return;
+
+	if (!(id = xmlGetProp (argument_data, "id"))) return;
+	set = g_tree_lookup (widget_db, id);
+
+	if (!set || !set->value_widget || (set->alias && !set->enabled))
+		return;
+
+	gtk_entry_set_text (GTK_ENTRY (set->value_widget), arg);
+
+	g_strfreev (args);
+}
+
 static void
 place_screensaver_properties (ScreensaverPrefsDialog *dialog,
 			      xmlNodePtr argument_data)
@@ -1349,6 +1654,12 @@ place_screensaver_properties (ScreensaverPrefsDialog *dialog,
 		else if (!strcmp (node->name, "select"))
 			read_select (dialog->widget_db, node, 
 				     dialog->cli_args_db);
+		else if (!strcmp (node->name, "string"))
+			read_string (dialog->widget_db, node, 
+				     dialog->cli_args_db, FALSE);
+		else if (!strcmp (node->name, "file"))
+			read_string (dialog->widget_db, node, 
+				     dialog->cli_args_db, TRUE);
 		else if (!strcmp (node->name, "hgroup"))
 			place_screensaver_properties (dialog, node);
 	}
