@@ -20,12 +20,12 @@
    Authors: Felix Bellaby, Owen Taylor */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#   include <config.h>
 #endif
 
 #include <string.h>
 #include <gnome.h>
-#include <libgnomeui/gnome-window-icon.h>
+#include <parser.h>
 #include "capplet-widget.h"
 #include "gsm-client-list.h"
 #include "gsm-protocol.h"
@@ -40,9 +40,6 @@ static gboolean trash_changes_revert;
 static gboolean logout_prompt;
 static gboolean logout_prompt_revert;
 
-static gboolean login_splash;
-static gboolean login_splash_revert;
-
 static GSList *startup_list;
 static GSList *startup_list_revert;
 
@@ -52,7 +49,6 @@ static GtkObject *protocol;
 static GtkWidget *capplet;
 static GtkWidget *trash_changes_button;
 static GtkWidget *logout_prompt_button;
-static GtkWidget *login_splash_button;
 
 static GtkWidget *clist;
 
@@ -109,9 +105,6 @@ capplet_build (void)
 
   logout_prompt = gnome_config_get_bool ("LogoutPrompt=true");
   logout_prompt_revert = logout_prompt;
-
-  login_splash = gnome_config_get_bool ("SplashScreen=true");
-  login_splash_revert = login_splash;
   gnome_config_pop_prefix ();
 
   startup_list = NULL;
@@ -153,13 +146,6 @@ capplet_build (void)
   alignment = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
   gtk_box_pack_start (GTK_BOX (util_vbox), alignment, FALSE, FALSE, 0);
 
-  login_splash_button = gtk_check_button_new_with_label
-    (_("Show splash screen on login"));
-  gtk_container_add (GTK_CONTAINER (alignment), login_splash_button);
-
-  alignment = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
-  gtk_box_pack_start (GTK_BOX (util_vbox), alignment, FALSE, FALSE, 0);
-
   logout_prompt_button = gtk_check_button_new_with_label
     (_("Prompt on logout"));
   gtk_container_add (GTK_CONTAINER (alignment), logout_prompt_button);
@@ -235,23 +221,62 @@ capplet_build (void)
 		      GTK_SIGNAL_FUNC (dirty_cb), capplet);
   gtk_signal_connect (GTK_OBJECT (logout_prompt_button), "toggled",
 		      GTK_SIGNAL_FUNC (dirty_cb), capplet);
-  gtk_signal_connect (GTK_OBJECT (login_splash_button), "toggled",
-		      GTK_SIGNAL_FUNC (dirty_cb), capplet);
+
   gtk_widget_show_all (capplet);
 }
 
+static void
+read_from_xml (xmlDocPtr doc) 
+{
+        xmlNodePtr root_node, node;
+
+        root_node = xmlDocGetRootElement (doc);
+        if (strcmp (root_node->name, "session-prefs")) return;
+
+        for (node = root_node->childs; node; node = node->next) {
+                if (!strcmp (node->name, "trash-mode"))
+                        trash_changes = FALSE;
+                else if (!strcmp (node->name, "logout-prompt"))
+                        logout_prompt = FALSE;
+                else if (!strcmp (node->name, "startup-programs"))
+                        startup_list = startup_list_read_from_xml (node);
+        }
+}
+
+static xmlDocPtr
+write_to_xml (void) 
+{
+        xmlDocPtr doc;
+        xmlNodePtr root_node;
+        char *str;
+
+        doc = xmlNewDoc ("1.0");
+        root_node = xmlNewDocNode (doc, NULL, "session-prefs", NULL);
+        xmlDocSetRootElement (doc, root_node);
+
+        if (trash_changes)
+                xmlNewChild (root_node, NULL, "trash-mode", NULL);
+
+        if (logout_prompt)
+                xmlNewChild (root_node, NULL, "logout-prompt", NULL);
+
+	xmlAddChild (root_node, startup_list_write_to_xml (startup_list));
+
+        return doc;
+}
+
 /* CAPPLET CALLBACKS */
 static void
 write_state (void)
 {
-  trash_changes = !GTK_TOGGLE_BUTTON (trash_changes_button)->active;
-  logout_prompt = GTK_TOGGLE_BUTTON (logout_prompt_button)->active;
-  login_splash  = GTK_TOGGLE_BUTTON (login_splash_button)->active;
+  if (trash_changes_button)
+    trash_changes = !GTK_TOGGLE_BUTTON (trash_changes_button)->active;
+  if (logout_prompt_button)
+    logout_prompt = GTK_TOGGLE_BUTTON (logout_prompt_button)->active;
   
   gnome_config_push_prefix (GSM_OPTION_CONFIG_PREFIX);
   gnome_config_set_bool ("TrashMode", trash_changes);
   gnome_config_set_bool ("LogoutPrompt", logout_prompt);
-  gnome_config_set_bool ("SplashScreen", login_splash);
   gnome_config_pop_prefix ();
 
   /* Set the trash mode in the session manager so it takes effect immediately.
@@ -268,7 +293,6 @@ revert_state (void)
 {
   trash_changes = trash_changes_revert;
   logout_prompt = logout_prompt_revert;
-  login_splash  = login_splash_revert;
 
   startup_list_free (startup_list);
   startup_list = startup_list_duplicate (startup_list_revert);
@@ -309,12 +333,10 @@ cancel (void)
 static void
 help (void)
 {
-  gchar *tmp;
-
-  tmp = gnome_help_file_find_file ("control-center", "session.html#STARTUP-PROGS");
-  if (tmp) {
-    gnome_help_goto (NULL, tmp);
-    g_free(tmp);
+  gchar* file = gnome_help_file_find_file(program_invocation_short_name, 
+					  "index.html");
+  if (file) {
+    gnome_help_goto (NULL, file);
   } else {
     GtkWidget *mbox;
     
@@ -348,8 +370,7 @@ update_gui (void)
 				!trash_changes);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (logout_prompt_button),
 				logout_prompt);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (login_splash_button),
-				login_splash);
+  
   startup_list_update_gui (startup_list, clist);
 }
 
@@ -409,6 +430,45 @@ browse_session_cb (void)
 
 /* STARTUP CODE */
 
+static void do_get_xml (void) 
+{
+        xmlDocPtr doc;
+
+	gnome_config_push_prefix (GSM_OPTION_CONFIG_PREFIX);
+	trash_changes = gnome_config_get_bool ("TrashMode=true");
+	logout_prompt = gnome_config_get_bool ("LogoutPrompt=true");
+	gnome_config_pop_prefix ();
+
+	startup_list = NULL;
+	startup_list = startup_list_read ("Default");
+
+        doc = write_to_xml ();
+        xmlDocDump (stdout, doc);
+}
+
+static void do_set_xml (void) 
+{
+        xmlDocPtr doc;
+	char *buffer;
+	int len = 0;
+
+	while (!feof (stdin)) {
+		if (!len) buffer = g_new (char, 16384);
+		else buffer = g_renew (char, buffer, len + 16384);
+		fread (buffer + len, 1, 16384, stdin);
+		len += 16384;
+	}
+
+	doc = xmlParseMemory (buffer, strlen (buffer));
+
+	protocol = gsm_protocol_new (gnome_master_client ());
+
+	if (!protocol) g_error ("Could not connect to gnome-session.");
+
+	read_from_xml (doc);
+        write_state ();
+}
+
 static gboolean warner = FALSE;
 
 static struct poptOption options[] = {
@@ -426,7 +486,7 @@ main (int argc, char *argv[])
 
   init_result = gnome_capplet_init("session-properties", VERSION, argc, argv,
 				   options, 0, NULL);
-  gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-session.png");
+
   gtk_signal_connect (GTK_OBJECT (gnome_master_client ()), "die",
 		      GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
   gnome_client_set_restart_style (gnome_master_client(), GNOME_RESTART_NEVER);
@@ -436,6 +496,13 @@ main (int argc, char *argv[])
     case 1:
       break;
 
+    case 3:
+	    do_get_xml ();
+	    return 0;
+    case 4:
+	    do_set_xml ();
+	    return 0;
+
     default:
 	    /* We need better error handling.  This prolly just means
 	     * one already exists.*/
