@@ -19,7 +19,10 @@
 
    Authors: Owen Taylor */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+#   include <config.h>
+#endif
+
 #include <ctype.h>
 #include <stdlib.h>
 #include <gnome.h>
@@ -412,3 +415,107 @@ startup_list_delete (GSList **sl, GtkWidget *clist)
       client_free (client);
     }
 }
+
+ManualClient *
+startup_client_read_from_xml (xmlNodePtr cl_node) 
+{
+	ManualClient *client;
+	xmlNodePtr node;
+	gchar *command;
+	GSList *arg_list_head = NULL, *arg_list_tail = NULL, *tmp;
+	int i;
+
+	if (strcmp (node->name, "program")) return NULL;
+
+	client = g_new0 (ManualClient, 1);
+
+	for (node = cl_node->childs; node; node = node->next) {
+		if (!strcmp (node->name, "priority"))
+			client->priority = atoi (xmlNodeGetContent (node));
+		else if (!strcmp (node->name, "command"))
+			command = xmlNodeGetContent (node);
+		else if (!strcmp (node->name, "argument")) {
+			arg_list_tail = g_slist_append 
+				(arg_list_tail, xmlNodeGetContent (node));
+			if (arg_list_head == NULL)
+				arg_list_head = arg_list_tail;
+			else
+				arg_list_tail = arg_list_tail->next;
+		}
+	}
+
+	client->argc = g_slist_length (arg_list_head) + 1;
+	client->argv = g_new0 (char *, client->argc);
+	client->argv[0] = command;
+
+	for (i = 1; i < client->argc; i++) {
+		g_assert (arg_list_head != NULL);
+		client->argv[i] = (char *) arg_list_head->data;
+		tmp = arg_list_head->next;
+		g_slist_free_1 (arg_list_head);
+		arg_list_head = tmp;
+	}
+
+	return client;
+}
+
+GSList *
+startup_list_read_from_xml (xmlNodePtr sl_node) 
+{
+	GSList *list_head = NULL, *list_tail = NULL;
+	ManualClient *client;
+	xmlNodePtr node;
+
+	if (strcmp (node->name, "startup-programs")) return NULL;
+
+	for (node = sl_node->childs; node; node = node->next) {
+		if (!strcmp (node->name, "program")) {
+			client = startup_client_read_from_xml (node);
+			list_tail = g_slist_append (list_tail, client);
+			if (list_head == NULL)
+				list_head = list_tail;
+			else
+				list_tail = list_tail->next;
+		}
+	}
+
+	return list_head;
+}
+
+xmlNodePtr
+startup_client_write_to_xml (ManualClient *client) 
+{
+	xmlNodePtr cl_node;
+	int i;
+	char *str;
+
+	if (client->argc == 0) return NULL;
+
+	cl_node = xmlNewNode (NULL, "program");
+
+	str = g_strdup_printf ("%d", client->priority);
+	xmlNewChild (cl_node, NULL, "priority", str);
+	g_free (str);
+
+	xmlNewChild (cl_node, NULL, "command", client->argv[0]);
+
+	for (i = 1; i < client->argc; i++)
+		xmlNewChild (cl_node, NULL, "argument", client->argv[i]);
+
+	return cl_node;
+}
+
+xmlNodePtr 
+startup_list_write_to_xml (GSList *sl) 
+{
+	xmlNodePtr sl_node, node;
+
+	sl_node = xmlNewNode (NULL, "startup-programs");
+
+	for (; sl; sl = sl->next) {
+		node = startup_client_write_to_xml ((ManualClient *) sl->data);
+		if (node) xmlAddChild (sl_node, node);
+	}
+
+	return sl_node;
+}
